<?php
namespace Teknisa\EST\Modules\Produto\Controllers;

use Teknisa\EST\Modules\Produto\Services\Produto as ProdutoService;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class ProdutoArvore extends \Teknisa\EST\Controllers\TeknisaCrud {

    protected $dataSourceName = "produto_para_arvore";
    protected $dataSourceNameToSave = "produto";
    protected $produtoService;

    public function __construct($crudService, $parameterBag, ProdutoService $produtoService) {
        parent::__construct($crudService, $parameterBag);
        $this->produtoService = $produtoService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions     = $filterCriteria->getConditions();

        //uncomment lines when Zeedhi Framework manage to get faster
        if(count($conditions) == 0) {
            $filterCriteria->addCondition("CDPRODNIVANTE", "IS_NULL", null);
        }
        
        parent::find($request, $response);
        
        $dataSets = $response->getDataSets();
        if (count($dataSets) > 0) {
            $rows = $dataSets[0]->getRows();

            $rows = array_map(function($row) {
                $row['_hasChild'] = $row['HASCHILD'] === 'true' ? true : false;
                unset($row['HASCHILD']);
                return $row;
            }, $rows);

            $response->setDataSets(array(new DataSet('produto_para_arvore', $rows)));
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {  
            $row['CDARVPROD'] = $this->produtoService->removeMask($row['CDARVPROD']);
            $this->produtoService->arvprodContainsProdnivante($row['CDARVPROD'], $row['CDPRODNIVANTE']);
            $row['CDNVPRODUTO'] = $this->produtoService->getCdnvprodutoByCdarvprod($row['CDARVPROD']);
            if ($row['__is_new'] === true) {
                $this->produtoService->existArvprod($row['CDARVPROD']);
                $row['CDPRODUTO'] = $this->produtoService->getNewCdproduto($row['CDARVPROD']);
            }
            $this->produtoService->correctLevelByArvprod($row['CDARVPROD'], $row['CDNVPRODUTO']);
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function suggestions(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $sugestedarvprod = $this->produtoService->getSugestedCdarvprod($row['PARENTCDPRODUTO'], $row['PARENTNVPRODUTO']);
        $return = array(
            'CDDARVPROD'=>$sugestedarvprod
        );
        
        $dataSet = new DataSet('sugestedArvprod', array($return));
        $response->setDataSets(array($dataSet));
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            $this->produtoService->canBeDeletedTree($row['CDARVPROD'], $row['CDPRODUTO']);
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}
